SystemOrganization addCategory: #FuelProgressUpdate!
SystemOrganization addCategory: #'FuelProgressUpdate-Tests'!

!FLSerializer methodsFor: '*FuelProgressUpdate-configuring' stamp: 'MartinDias 5/8/2012 08:16'!
showProgress

	serializationFactory := [:anObject :anEncoder | 
		(FLProgressSerialization 
				with: anEncoder
				root: anObject 
				analyzer: self analyzer showProgress)
				run;
				yourself ]! !

FLBasicSerializationTest subclass: #FLProgressTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelProgressUpdate-Tests'!

!FLProgressTest classSide methodsFor: 'as yet unclassified' stamp: 'MartinDias 5/8/2012 08:36'!
shouldInheritSelectors 
	^ true! !

!FLProgressTest methodsFor: 'running' stamp: 'MartinDias 5/8/2012 00:39'!
materialization

	| result hasShownProgress |
	hasShownProgress := false.

	result := [ super materialization ]
		on: ProgressInitiationException
		do: [ :ex | 
			ex sendNotificationsTo: [ :min :max :curr | 
				hasShownProgress := true ] ].

	self assert: hasShownProgress.
	^ result
	! !

!FLProgressTest methodsFor: 'running' stamp: 'MartinDias 5/8/2012 00:38'!
serialize: anObject

	| hasShownProgress result |
	hasShownProgress := false.
	
	result := [ super serialize: anObject ]
		on: ProgressInitiationException
		do: [ :ex | 
			ex sendNotificationsTo: [ :min :max :curr | 
				hasShownProgress := true ] ].
	
	self assert: hasShownProgress.
	^ result! !

!FLProgressTest methodsFor: 'running' stamp: 'MartinDias 5/8/2012 00:37'!
setUpMaterializer
	super setUpMaterializer.
	self materializer showProgress.! !

!FLProgressTest methodsFor: 'running' stamp: 'MartinDias 5/8/2012 00:37'!
setUpSerializer
	super setUpSerializer.
	self serializer showProgress.! !

!FLProgressTest methodsFor: 'tests' stamp: 'MartinDias 5/8/2012 08:41'!
testInterval
	"Overriden because some intervals in super implementation are too big for converting to IdentitySet as currently needs FLProgressAnalysis."

	self assertSerializationEqualityOf: (-10 to: 10 by: 5).! !

FLAnalysis subclass: #FLProgressAnalysis
	instanceVariableNames: 'progressListener remainingCheckpoints totalNumberOfCheckpoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelProgressUpdate'!

!FLProgressAnalysis commentStamp: '<historical>' prior: 0!
I am an Analysis which notifies progress.
!

!FLProgressAnalysis methodsFor: 'progress handling' stamp: 'MartinDias 5/25/2012 21:12'!
checkpoints

	^ IdentitySet withAll: (root isCollection 
		ifTrue: [ self checkpointsOldPharoWorkaround ]
		ifFalse: [ #() ])! !

!FLProgressAnalysis methodsFor: 'progress handling' stamp: 'MartinDias 5/26/2012 01:17'!
checkpointsOldPharoWorkaround
	"In old Pharo versions Sets can't include nil."

	self flag: #todo. "fix in another way"
	^ SystemVersion current highestUpdate < 13000
		ifTrue: [ root asArray copyWithout: nil ]
		ifFalse: [ root ]! !

!FLProgressAnalysis methodsFor: 'mapping' stamp: 'MartinDias 5/8/2012 08:03'!
mapAndTrace: anObject

	self notifyAnalized: anObject.	
	^ super mapAndTrace: anObject.! !

!FLProgressAnalysis methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:02'!
notifyAnalized: anObject

	remainingCheckpoints
		remove: anObject
		ifAbsent: [ ^self ].
	progressListener value: self progress! !

!FLProgressAnalysis methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 07:59'!
progress

	^ 100 * (1 - (remainingCheckpoints size / totalNumberOfCheckpoints))! !

!FLProgressAnalysis methodsFor: 'analyzing' stamp: 'MartinDias 5/25/2012 19:31'!
run

	^ self title
		displayProgressFrom: 0 to: 100 
		during: [ :aProgressListener | 
			progressListener := aProgressListener.
			remainingCheckpoints := self checkpoints.
			totalNumberOfCheckpoints := remainingCheckpoints size.
			super run ]! !

!FLProgressAnalysis methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:07'!
title
	^ 'Analyzing graph'! !

!FLAnalyzer methodsFor: '*FuelProgressUpdate-configuring' stamp: 'MartinDias 5/8/2012 08:14'!
showProgress

	analysisFactory := [:anObject | 
		(FLProgressAnalysis 
			newWith: self firstInMapperChain 
			root: anObject)
			run;
			yourself ]! !

FLMaterialization subclass: #FLProgressMaterialization
	instanceVariableNames: 'progressHandler progressListener numberOfObjectsAlreadyMaterialized numberOfObjectsToMaterialize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelProgressUpdate'!

!FLProgressMaterialization commentStamp: '<historical>' prior: 0!
I am a Materialization which notifies progress.
!

!FLProgressMaterialization methodsFor: 'private' stamp: 'MartinDias 5/8/2012 08:21'!
instancesStep

	numberOfObjectsAlreadyMaterialized := 0.
	numberOfObjectsToMaterialize := self objects capacity.
	^ super instancesStep.
! !

!FLProgressMaterialization methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:25'!
notifyMaterializedAll: aCollection
	
	numberOfObjectsAlreadyMaterialized := numberOfObjectsAlreadyMaterialized + aCollection size.
	progressListener value: self progress.! !

!FLProgressMaterialization methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:27'!
progress

	^ 100 * numberOfObjectsAlreadyMaterialized / numberOfObjectsToMaterialize! !

!FLProgressMaterialization methodsFor: 'private' stamp: 'MartinDias 5/8/2012 08:25'!
registerAll: aCollection

	self notifyMaterializedAll: aCollection.
	^ super registerAll: aCollection! !

!FLProgressMaterialization methodsFor: 'materializing' stamp: 'MartinDias 5/8/2012 08:17'!
run

	^ self title
		displayProgressFrom: 0 to: 100 
		during: [ :aProgressListener | 
			progressListener := aProgressListener.
			super run ]
! !

!FLProgressMaterialization methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:27'!
title
	^'Reading from stream'! !

!FLMaterializer methodsFor: '*FuelProgressUpdate-configuring' stamp: 'MartinDias 5/8/2012 08:28'!
showProgress

	materializationFactory := [:aDecoder | 
		(FLProgressMaterialization 
			with: aDecoder)
			run;
			yourself ]! !

FLSerialization subclass: #FLProgressSerialization
	instanceVariableNames: 'progressHandler numberOfObjectsAlreadySerialized numberOfObjectsToSerialize progressListener'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelProgressUpdate'!

!FLProgressSerialization commentStamp: '<historical>' prior: 0!
I am a Serialization which notifies progress.
!

!FLProgressSerialization methodsFor: 'private' stamp: 'MartinDias 5/8/2012 08:12'!
clusterInstancesStepOf: aCluster

	self notifySerialized: aCluster.
	^ super clusterInstancesStepOf: aCluster.
! !

!FLProgressSerialization methodsFor: 'private' stamp: 'MartinDias 5/8/2012 08:10'!
headerStep

	numberOfObjectsAlreadySerialized := 0.
	numberOfObjectsToSerialize := encoder objectCount.
	^ super headerStep.! !

!FLProgressSerialization methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:13'!
notifySerialized: aCluster

	numberOfObjectsAlreadySerialized := numberOfObjectsAlreadySerialized + aCluster objects size.
	progressListener value: self progress.! !

!FLProgressSerialization methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:11'!
progress

	^ 100 * numberOfObjectsAlreadySerialized / numberOfObjectsToSerialize! !

!FLProgressSerialization methodsFor: 'serializing' stamp: 'MartinDias 5/8/2012 08:13'!
run

	^ self title
		displayProgressFrom: 0 to: 100 
		during: [ :aProgressListener | 
			progressListener := aProgressListener.
			super run ]
! !

!FLProgressSerialization methodsFor: 'progress handling' stamp: 'MartinDias 5/8/2012 08:06'!
title
	^'Writing to stream'! !

Object subclass: #ManifestFuelProgressUpdate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelProgressUpdate'!

!ManifestFuelProgressUpdate classSide methodsFor: 'meta data' stamp: 'MarianoMartinezPeck 4/21/2012 17:08'!
rejectClasses
^ #()! !

!ManifestFuelProgressUpdate classSide methodsFor: 'meta data' stamp: 'MarianoMartinezPeck 4/21/2012 17:08'!
rejectRules
^ #()! !

!ManifestFuelProgressUpdate classSide methodsFor: 'meta data' stamp: 'MarianoMartinezPeck 4/21/2012 17:26'!
ruleCodeCruftLeftInMethodsRuleV1FalsePositive
^ #(#('(RGMethodDefinition className: ''FLProgressSerialization class'' selector: #defaultProgressBarFactory isMetaSide: true)' #'2012-04-21T17:26:41.162000001+02:00') )! !

!ManifestFuelProgressUpdate classSide methodsFor: 'meta data' stamp: 'MarianoMartinezPeck 4/21/2012 17:34'!
ruleInconsistentMethodClassificationRuleV1TruePositive
^ #(#('(RGMethodDefinition className: ''FLProgressSerialization'' selector: #clusterInstancesStepOf: isMetaSide: false)' #'2012-04-21T17:34:52.439000001+02:00') #('(RGMethodDefinition className: ''FLProgressSerialization'' selector: #headerStep isMetaSide: false)' #'2012-04-21T17:34:52.488000001+02:00') #('(RGMethodDefinition className: ''FLProgressAnalysis'' selector: #run isMetaSide: false)' #'2012-04-21T17:34:52.508+02:00') )! !

!ManifestFuelProgressUpdate classSide methodsFor: 'meta data' stamp: 'MarianoMartinezPeck 4/21/2012 23:00'!
ruleVariableReferencedOnceRuleV1TruePositive
^ #(#('(RGMethodDefinition className: ''FLProgressMaterializationTest'' selector: #serializeAndMaterialize: isMetaSide: false)' #'2012-04-21T23:00:30.335+02:00') )! !
